@@ -2,9 +2,11 @@
 //! manual specifications for column count, column widths, and border styling.
 //!
 //! * [`CompactTable`] is a [`Table`] alternative that trades off reduced
-//! flexibility for improved performance.
+//!   flexibility for improved performance.
 
-use tabled::{settings::Style, tables::CompactTable};
+#![allow(unused_variables)]
+
+use tabled::{settings::style::Style, tables::CompactTable};
 
 fn main() {
     let data = [
@@ -13,11 +15,11 @@ fn main() {
         ["Manjaro", "Arch", "true"],
     ];
 
-    let _table = CompactTable::new(data)
+    let table = CompactTable::new(data)
         .columns(3)
         .width([7, 5, 5])
         .with(Style::markdown());
 
     #[cfg(feature = "std")]
-    println!("{}", _table.to_string());
+    println!("{}", table.to_string());
 }
