@@ -115,18 +115,21 @@ pub struct PoolTable {
 
 impl PoolTable {
     /// Creates a [`PoolTable`] out from a record iterator.
-    pub fn new<I: IntoRecords>(iter: I) -> Self {
+    pub fn new<I>(iter: I) -> Self
+    where
+        I: IntoRecords,
+        I::Cell: AsRef<str>,
+    {
         let value = TableValue::Column(
             iter.iter_rows()
                 .into_iter()
                 .map(|row| {
-                    TableValue::Row(
-                        row.into_iter()
-                            .map(|cell| cell.as_ref().to_string())
-                            .map(TableValue::Cell)
-                            .collect(),
-                    )
+                    row.into_iter()
+                        .map(|cell| cell.as_ref().to_string())
+                        .map(TableValue::Cell)
+                        .collect()
                 })
+                .map(TableValue::Row)
                 .collect(),
         );
 
@@ -169,7 +172,7 @@ impl PoolTable {
     /// ```
     pub fn with<O>(&mut self, option: O) -> &mut Self
     where
-        O: TableOption<EmptyRecords, PoolTableDimension, CompactMultilineConfig>,
+        O: TableOption<EmptyRecords, CompactMultilineConfig, PoolTableDimension>,
     {
         let mut records = EmptyRecords::default();
         option.change(&mut records, &mut self.config, &mut self.dims);
@@ -209,23 +212,25 @@ fn configure_grid() -> CompactMultilineConfig {
     let pad = Sides::new(
         Indent::spaced(1),
         Indent::spaced(1),
-        Indent::default(),
-        Indent::default(),
+        Indent::zero(),
+        Indent::zero(),
     );
 
-    CompactMultilineConfig::default()
-        .set_padding(pad)
-        .set_alignment_horizontal(AlignmentHorizontal::Left)
-        .set_borders(*Style::ascii().get_borders())
+    let mut cfg = CompactMultilineConfig::new();
+    cfg.set_padding(pad);
+    cfg.set_alignment_horizontal(AlignmentHorizontal::Left);
+    cfg.set_borders(Style::ascii().get_borders());
+
+    cfg
 }
 
-impl<R, C> TableOption<R, PoolTableDimension, C> for PoolTableDimension {
+impl<R, C> TableOption<R, C, PoolTableDimension> for PoolTableDimension {
     fn change(self, _: &mut R, _: &mut C, dimension: &mut PoolTableDimension) {
         *dimension = self;
     }
 }
 
-impl<R, D> TableOption<R, D, CompactMultilineConfig> for CompactMultilineConfig {
+impl<R, D> TableOption<R, CompactMultilineConfig, D> for CompactMultilineConfig {
     fn change(self, _: &mut R, config: &mut CompactMultilineConfig, _: &mut D) {
         *config = self;
     }
@@ -234,24 +239,20 @@ impl<R, D> TableOption<R, D, CompactMultilineConfig> for CompactMultilineConfig
 mod print {
     use std::{cmp::max, collections::HashMap, iter::repeat};
 
-    use papergrid::{
-        color::StaticColor,
-        config::{Border, Borders},
-        util::string::string_width_multiline,
-    };
-
     use crate::{
         builder::Builder,
         grid::{
+            ansi::ANSIStr,
             config::{
-                AlignmentHorizontal, AlignmentVertical, ColoredConfig, CompactMultilineConfig,
-                Indent, Offset, Sides,
+                AlignmentHorizontal, AlignmentVertical, Border, Borders, CompactMultilineConfig,
+                Indent, Sides,
+            },
+            dimension::{DimensionPriority, PoolTableDimension},
+            util::string::{
+                count_lines, get_line_width, get_lines, get_text_dimension, get_text_width,
             },
-            dimension::{Dimension, DimensionPriority, Estimate, PoolTableDimension},
-            records::Records,
-            util::string::{count_lines, get_lines, string_dimension, string_width},
         },
-        settings::{Padding, Style, TableOption},
+        settings::{Padding, Style},
     };
 
     use super::TableValue;
@@ -318,7 +319,7 @@ mod print {
             || margin.left.size > 0
             || margin.right.size > 0;
         if has_margin {
-            let color = convert_border_colors(cfg.get_margin_color());
+            let color = convert_border_colors(*cfg.get_margin_color());
             table = set_margin(&table, *margin, color);
         }
 
@@ -352,7 +353,7 @@ mod print {
     }
 
     fn generate_table_column(
-        list: &Vec<TableValue>,
+        list: &[TableValue],
         cfg: &CompactMultilineConfig,
         dims: &Dimensions,
         priority: PoolTableDimension,
@@ -428,7 +429,7 @@ mod print {
             intersections_horizontal = data.intersections_horizontal;
             next_intersections_vertical.extend(data.intersections_vertical);
 
-            let _ = builder.push_record([data.content]);
+            builder.push_record([data.content]);
         }
 
         let table = builder
@@ -441,7 +442,7 @@ mod print {
     }
 
     fn generate_table_row(
-        list: &Vec<TableValue>,
+        list: &[TableValue],
         cfg: &CompactMultilineConfig,
         dims: &Dimensions,
         priority: PoolTableDimension,
@@ -521,13 +522,14 @@ mod print {
             buf.push(value);
         }
 
-        let mut b = Builder::with_capacity(1);
-        let _ = b.hint_column_size(buf.len()).push_record(buf);
-        let table = b
-            .build()
-            .with(Style::empty())
-            .with(Padding::zero())
-            .to_string();
+        let mut builder = Builder::with_capacity(1, buf.len());
+        builder.push_record(buf);
+
+        let mut table = builder.build();
+        let _ = table.with(Style::empty());
+        let _ = table.with(Padding::zero());
+
+        let table = table.to_string();
 
         CellData::new(table, new_intersections_horizontal, intersections_vertical)
     }
@@ -555,13 +557,12 @@ mod print {
         let halignment = cfg.get_alignment_horizontal();
         let valignment = cfg.get_alignment_vertical();
         let pad = cfg.get_padding();
-        let pad_color = cfg.get_padding_color();
-        let pad_color = convert_border_colors(pad_color);
-        let lines_alignemnt = cfg.get_formatting().allow_lines_alignment;
+        let pad_color = convert_border_colors(*cfg.get_padding_color());
+        let lines_alignment = cfg.get_formatting().allow_lines_alignment;
 
         let mut borders = *cfg.get_borders();
 
-        let bottom_intesection = cfg.get_borders().bottom_intersection.unwrap_or(' ');
+        let bottom_intersection = cfg.get_borders().bottom_intersection.unwrap_or(' ');
         let mut horizontal_splits = short_splits(&mut ctx.intersections_horizontal, ctx.size.width);
         squash_splits(&mut horizontal_splits);
 
@@ -590,7 +591,7 @@ mod print {
             border,
             border_color,
             &horizontal_splits,
-            bottom_intesection,
+            bottom_intersection,
             ctx.size.width,
         );
 
@@ -626,9 +627,9 @@ mod print {
             line_index += 1;
         }
 
-        if lines_alignemnt {
+        if lines_alignment {
             for line in get_lines(text) {
-                let line_width = string_width(&line);
+                let line_width = get_line_width(&line);
                 let (left, right) = indent_horizontal(halignment, width, line_width);
 
                 if border.has_left() {
@@ -656,11 +657,11 @@ mod print {
                 line_index += 1;
             }
         } else {
-            let text_width = string_width_multiline(text);
+            let text_width = get_text_width(text);
             let (left, _) = indent_horizontal(halignment, width, text_width);
 
             for line in get_lines(text) {
-                let line_width = string_width(&line);
+                let line_width = get_line_width(&line);
                 let right = width - line_width - left;
 
                 if border.has_left() {
@@ -726,7 +727,7 @@ mod print {
         buf
     }
 
-    fn print_chars(buf: &mut String, c: char, color: Option<StaticColor>, width: usize) {
+    fn print_chars(buf: &mut String, c: char, color: Option<ANSIStr<'static>>, width: usize) {
         match color {
             Some(color) => {
                 buf.push_str(color.get_prefix());
@@ -737,7 +738,7 @@ mod print {
         }
     }
 
-    fn print_char(buf: &mut String, c: char, color: Option<StaticColor>) {
+    fn print_char(buf: &mut String, c: char, color: Option<ANSIStr<'static>>) {
         match color {
             Some(color) => {
                 buf.push_str(color.get_prefix());
@@ -751,8 +752,8 @@ mod print {
     fn print_line(
         buf: &mut String,
         border: Border<char>,
-        border_color: Border<StaticColor>,
-        color: Option<StaticColor>,
+        border_color: Border<ANSIStr<'static>>,
+        color: Option<ANSIStr<'static>>,
         c: char,
         width: usize,
     ) {
@@ -774,7 +775,7 @@ mod print {
     fn print_top_line(
         buf: &mut String,
         border: Border<char>,
-        color: Border<StaticColor>,
+        color: Border<ANSIStr<'static>>,
         splits: &[usize],
         split_char: char,
         width: usize,
@@ -783,7 +784,7 @@ mod print {
             return;
         }
 
-        let mut used_color: Option<StaticColor> = None;
+        let mut used_color: Option<ANSIStr<'static>> = None;
 
         if border.has_left() {
             if let Some(color) = color.left_top_corner {
@@ -855,14 +856,14 @@ mod print {
     fn print_bottom_line(
         buf: &mut String,
         border: Border<char>,
-        color: Border<StaticColor>,
+        color: Border<ANSIStr<'static>>,
         width: usize,
     ) {
         if !border.has_bottom() {
             return;
         }
 
-        let mut used_color: Option<StaticColor> = None;
+        let mut used_color: Option<ANSIStr<'static>> = None;
 
         if border.has_left() {
             if let Some(color) = color.left_bottom_corner {
@@ -981,103 +982,6 @@ mod print {
         }
     }
 
-    struct ConfigCell(PrintContext);
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for ConfigCell {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            {
-                // we set a horizontal lines to borders to not complicate logic with cleaning it
-
-                let mut borders = *cfg.get_borders();
-                if let Some(line) = cfg.get_horizontal_line(0) {
-                    borders.top = line.main;
-                    borders.top_left = line.left;
-                    borders.top_right = line.right;
-                }
-
-                if let Some(line) = cfg.get_horizontal_line(1) {
-                    borders.bottom = line.main;
-                    borders.bottom_left = line.left;
-                    borders.bottom_right = line.right;
-                }
-
-                cfg.clear_theme();
-                cfg.set_borders(borders);
-            }
-
-            let mut ctx = self.0;
-
-            let has_vertical = cfg.get_borders().has_left();
-            if !ctx.intersections_horizontal.is_empty() && has_vertical {
-                let mut splits = short_splits(&mut ctx.intersections_horizontal, ctx.size.width);
-                squash_splits(&mut splits);
-
-                let c = cfg.get_borders().bottom_intersection.unwrap_or(' ');
-                cfg_set_top_chars(cfg, &splits, c)
-            }
-
-            let has_horizontal = cfg.get_borders().has_top();
-            if !ctx.intersections_vertical.is_empty() && has_horizontal {
-                let mut splits = short_splits(&mut ctx.intersections_vertical, ctx.size.height);
-                squash_splits(&mut splits);
-
-                let c = cfg.get_borders().right_intersection.unwrap_or(' ');
-                cfg_set_left_chars(cfg, &splits, c)
-            }
-
-            let mut borders = *cfg.get_borders();
-
-            // set top_left
-            {
-                if ctx.kv && ctx.kv_is_first {
-                    borders.top_left = borders.top_intersection;
-                }
-
-                if ctx.kv && !ctx.kv_is_first {
-                    borders.top_left = borders.intersection;
-                }
-
-                if ctx.kv && ctx.list && !ctx.list_is_first {
-                    borders.top_left = borders.left_intersection;
-                }
-
-                if ctx.is_first_col && !ctx.is_first_row {
-                    borders.top_left = borders.left_intersection;
-                }
-
-                if ctx.lean_top {
-                    borders.top_left = borders.top_intersection;
-                }
-
-                if ctx.top_left {
-                    borders.top_left = borders.left_intersection;
-                }
-
-                if ctx.top_intersection {
-                    borders.top_left = borders.intersection;
-                }
-            }
-
-            if ctx.is_last_col && !ctx.is_first_row {
-                borders.top_right = borders.right_intersection;
-            }
-
-            if !ctx.is_first_col && ctx.is_last_row {
-                borders.bottom_left = borders.bottom_intersection;
-            }
-
-            if !ctx.is_last_row || ctx.no_bottom {
-                cfg_no_bottom_borders(&mut borders);
-            }
-
-            if ctx.no_right {
-                cfg_no_right_borders(&mut borders);
-            }
-
-            cfg.set_borders(borders);
-        }
-    }
-
     fn cfg_no_bottom_borders(borders: &mut Borders<char>) {
         borders.bottom = None;
         borders.bottom_intersection = None;
@@ -1094,231 +998,6 @@ mod print {
         borders.vertical = None;
     }
 
-    fn cfg_set_top_chars(cfg: &mut ColoredConfig, list: &[usize], c: char) {
-        for &split in list {
-            let offset = split;
-            cfg.set_horizontal_char((0, 0), c, Offset::Begin(offset));
-        }
-    }
-
-    fn cfg_set_left_chars(cfg: &mut ColoredConfig, list: &[usize], c: char) {
-        for &offset in list {
-            cfg.set_vertical_char((0, 0), c, Offset::Begin(offset));
-        }
-    }
-
-    struct NoTopBorders;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for NoTopBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top = None;
-            borders.top_intersection = None;
-            borders.top_left = None;
-            borders.top_right = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct NoBottomBorders;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for NoBottomBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom = None;
-            borders.bottom_intersection = None;
-            borders.bottom_left = None;
-            borders.bottom_right = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct NoRightBorders;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for NoRightBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_right = None;
-            borders.bottom_right = None;
-            borders.right = None;
-            borders.right_intersection = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct NoLeftBorders;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for NoLeftBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = None;
-            borders.bottom_left = None;
-            borders.left = None;
-            borders.left_intersection = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct TopLeftChangeTopIntersection;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for TopLeftChangeTopIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = borders.top_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct TopLeftChangeIntersection;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for TopLeftChangeIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = borders.intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct TopLeftChangeToLeft;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for TopLeftChangeToLeft {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = borders.left_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct TopRightChangeToRight;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for TopRightChangeToRight {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_right = borders.right_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct BottomLeftChangeSplit;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for BottomLeftChangeSplit {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_left = borders.left_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct BottomLeftChangeSplitToIntersection;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for BottomLeftChangeSplitToIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_left = borders.intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct BottomRightChangeToRight;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for BottomRightChangeToRight {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_right = borders.right_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct BottomLeftChangeToBottomIntersection;
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for BottomLeftChangeToBottomIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_left = borders.bottom_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    struct SetBottomChars<'a>(&'a [usize], char);
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for SetBottomChars<'_>
-    where
-        R: Records,
-        for<'a> &'a R: Records,
-        for<'a> D: Dimension + Estimate<&'a R, ColoredConfig>,
-    {
-        fn change(self, records: &mut R, cfg: &mut ColoredConfig, dims: &mut D) {
-            dims.estimate(&*records, cfg);
-
-            let table_width = (0..records.count_columns())
-                .map(|col| dims.get_width(col))
-                .sum::<usize>()
-                + cfg.count_vertical(records.count_columns());
-            let mut current_width = 0;
-
-            for pos in self.0 {
-                current_width += pos;
-                if current_width > table_width {
-                    break;
-                }
-
-                let split_char = self.1;
-                cfg.set_horizontal_char((1, 0), split_char, Offset::Begin(current_width));
-
-                current_width += 1;
-            }
-        }
-    }
-
-    struct SetTopChars<'a>(&'a [usize], char);
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for SetTopChars<'_> {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            for &split in self.0 {
-                let offset = split;
-                cfg.set_horizontal_char((0, 0), self.1, Offset::Begin(offset));
-            }
-        }
-    }
-
-    struct SetLeftChars<'a>(&'a [usize], char);
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for SetLeftChars<'_> {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            for &offset in self.0 {
-                cfg.set_vertical_char((0, 0), self.1, Offset::Begin(offset));
-            }
-        }
-    }
-
-    struct GetTopIntersection(char);
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for &mut GetTopIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            self.0 = cfg.get_borders().top_intersection.unwrap_or(' ');
-        }
-    }
-
-    struct GetBottomIntersection(char);
-
-    impl<R, D> TableOption<R, D, ColoredConfig> for &mut GetBottomIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            self.0 = cfg.get_borders().bottom_intersection.unwrap_or(' ');
-        }
-    }
-
     #[derive(Debug, Default)]
     struct Dimensions {
         all: HashMap<usize, Dim>,
@@ -1444,7 +1123,7 @@ mod print {
     }
 
     fn str_dimension(text: &str, cfg: &CompactMultilineConfig) -> Dim {
-        let (count_lines, width) = string_dimension(text);
+        let (count_lines, width) = get_text_dimension(text);
         let w = width + get_padding_horizontal(cfg);
         let h = count_lines + get_padding_vertical(cfg);
         Dim::new(w, h)
@@ -1559,13 +1238,17 @@ mod print {
         splits.iter_mut().enumerate().for_each(|(i, s)| *s += i);
     }
 
-    fn set_margin(table: &str, margin: Sides<Indent>, color: Sides<Option<StaticColor>>) -> String {
+    fn set_margin(
+        table: &str,
+        margin: Sides<Indent>,
+        color: Sides<Option<ANSIStr<'static>>>,
+    ) -> String {
         if table.is_empty() {
             return String::new();
         }
 
         let mut buf = String::new();
-        let width = string_width_multiline(table);
+        let width = get_text_width(table);
         let top_color = color.top;
         let bottom_color = color.bottom;
         let left_color = color.left;
@@ -1596,12 +1279,14 @@ mod print {
         buf
     }
 
-    fn convert_border_colors(pad_color: Sides<StaticColor>) -> Sides<Option<StaticColor>> {
+    fn convert_border_colors(
+        pad_color: Sides<ANSIStr<'static>>,
+    ) -> Sides<Option<ANSIStr<'static>>> {
         Sides::new(
-            (!pad_color.left.is_empty()).then(|| pad_color.left),
-            (!pad_color.right.is_empty()).then(|| pad_color.right),
-            (!pad_color.top.is_empty()).then(|| pad_color.top),
-            (!pad_color.bottom.is_empty()).then(|| pad_color.bottom),
+            (!pad_color.left.is_empty()).then_some(pad_color.left),
+            (!pad_color.right.is_empty()).then_some(pad_color.right),
+            (!pad_color.top.is_empty()).then_some(pad_color.top),
+            (!pad_color.bottom.is_empty()).then_some(pad_color.bottom),
         )
     }
 }
